@@ -3,6 +3,8 @@ package DNS::Unbound::AsyncQuery;
 use strict;
 use warnings;
 
+use parent qw( Promise::ES6 );
+
 =encoding utf-8
 
 =head1 NAME
@@ -23,8 +25,7 @@ DNS::Unbound::AsyncQuery
 =head1 DESCRIPTION
 
 This object represents the result of an asynchronous L<DNS::Unbound> query.
-It implements a standard promise interface
-and also provides a cancellation mechanism.
+It subclasses L<Promise::ES6> but provides a cancellation mechanism.
 
 The promise resolves with a L<DNS::Unbound::Result> instance.
 It rejects with a L<DNS::Unbound::X::ResolveError> instance
@@ -45,8 +46,8 @@ our $CANCEL_CR;
 
 =head1 METHODS
 
-In addition to the C<then()>, C<catch()>, and C<finally()> methods—see
-L<Promise::ES6> if you’re unsure of how to use those—this class provides:
+In addition to the methods inherited from L<Promise::ES6>, this
+class provides:
 
 =cut
 
@@ -54,8 +55,6 @@ L<Promise::ES6> if you’re unsure of how to use those—this class provides:
 
 Cancels an in-progress DNS query. Returns nothing.
 
-B<NOTE:> This will leave the promise I<unresolved>.
-
 =cut
 
 sub cancel {
@@ -65,7 +64,6 @@ sub cancel {
 
     if (!$dns_hr->{'fulfilled'}) {
         if (my $ctx = delete $dns_hr->{'ctx'}) {
-            delete $dns_hr->{'queries_lookup'}{ $dns_hr->{'id'} };
             $CANCEL_CR->( $ctx, $dns_hr->{'id'} );
         }
     }
@@ -76,55 +74,32 @@ sub cancel {
 # Leaving undocumented since as far as the caller is concerned
 # this method is identical to the parent class’s.
 sub then {
+    my $self = shift;
 
-    # We need to use the promise backend’s then(), but that backend isn’t
-    # a parent class of *this* class, so we can’t call SUPER::then().
-    # Thus, the backend bridge module aliases the backend then() to
-    # _then(), and we call into that.
+    my $new = $self->SUPER::then(@_);
 
-    my $new = $_[0]->_then(@_[1, 2]);
-
-    $new->_set_dns( $_[0]->_get_dns() );
+    $new->_set_dns( $self->_get_dns() );
 
     return $new;
 }
 
-# Not all promise implementations define these as wrappers around then().
-# So let’s be explicit about it:
-sub catch {
-    return $_[0]->then( undef, $_[1] );
-}
-
-sub finally {
-    return $_[0]->then( $_[1], $_[1] );
-}
-
 # ----------------------------------------------------------------------
 # Interfaces for DNS::Unbound to interact with the query’s DNS state.
 # Nothing external should call these other than DNS::Unbound.
 
-my %QUERY_OBJ_DNS;
-
 sub _set_dns {
     my ($self, $dns_hr) = @_;
-
-    $QUERY_OBJ_DNS{$self} = $dns_hr;
-
+    $self->{'_dns'} = $dns_hr;
     return $self;
 }
 
 sub _get_dns {
-    return $QUERY_OBJ_DNS{$_[0]};
+    return $_[0]->{'_dns'};
 }
 
-sub DESTROY {
-    my $self = shift;
-
-    delete $QUERY_OBJ_DNS{$self};
-
-    $self->SUPER::DESTROY() if $self->can('SUPER::DESTROY');
-
-    return;
+sub _forget_dns {
+    delete $_[0]->{'_dns'};
+    return $_[0];
 }
 
 1;
