@@ -26,11 +26,11 @@ The following all return scalars:
 
 =item * C<rcode()>, C<nxdomain()>, C<havedata()>, C<canonname()>
 
-=item * C<secure()>, C<bogus()>, C<why_bogus()>, C<answer_packet>
+=item * C<secure()>, C<bogus()>, C<why_bogus()>
 
 =back
 
-C<data()> returns an array reference of byte strings that contain the query
+C<data()> returns an array reference of strings that contain the query
 result in DNS-native RDATA encoding.
 
 =cut
@@ -51,7 +51,6 @@ use Class::XSAccessor {
         bogus => 'bogus',
         why_bogus => 'why_bogus',
         ttl => 'ttl',
-        answer_packet => 'answer_packet',
     },
 };
 
@@ -59,16 +58,6 @@ use Class::XSAccessor {
 
 =head2 $objs_ar = I<OBJ>->to_net_dns_rrs()
 
-B<IMPORTANT:> This method is DEPRECATED and will be withdrawn in a
-forthcoming version. Please migrate to the following logic instead
-(assuming an instance of this class in C<$result>):
-
-    my $packet = Net::DNS::Packet->new( \$result->answer_packet() );
-
-… which will yield a L<Net::DNS::Packet> instance.
-
-The DEPRECATED method’s documentation follows:
-
 The C<data()> accessor’s return values are raw RDATA. Your application
 likely prefers to work with parsed DNS data, though. This method facilitates
 that by loading L<Net::DNS::RR> and returning a reference to an array of
@@ -80,6 +69,10 @@ character strings, you could do:
 
     @cstrings = map { $_->txtdata() } @{ $result->to_net_dns_rrs() }
 
+(NB: It might be ideal to return a single L<Net::DNS::Packet> instance
+rather than the array reference, but C<struct ub_result> doesn’t expose
+enough of the underlying query’s DNS details for that to make sense.)
+
 =cut
 
 sub to_net_dns_rrs {
